class JWP_Bumper extends JWP_TableObject;

/* How many points the bumper awards on a collision with the ball. */
var int Points;

/* Executed on any collision.                                                 */
/*----------------------------------------------------------------------------*/
event RigidBodyCollision( PrimitiveComponent HitComponent, PrimitiveComponent OtherComponent, const out CollisionImpactData RigidCollisionData, int ContactIndex )
{

    /* Run the superclass's RigidBodyCollision function.  In this instance, TableObject will play a sound. */
    Super.RigidBodyCollision( HitComponent, OtherComponent, RigidCollisionData, ContactIndex );

    /* Score the appropiate number of points. */
    JWP_Gameinfo( WorldInfo.Game ).ScorePoints( Points );

    /* Push the lid down. */
    PhysicsTarget = PhysicsOpen;

}

/* Executed on every frame.                                                   */
/*----------------------------------------------------------------------------*/
simulated event Tick( float DeltaSeconds )
{

    local vector Build;

    /* Execute the superclass's Tick - in this case TableObject which will update the physics. */
    Super.Tick( DeltaSeconds );

    /* Use the physics data to build a cap position. */
    Build.x = 0;
    Build.y = 0;
    Build.z = PhysicsCurrent;

    /* Use it to translate our mesh. */
    StaticMeshComponent.SetTranslation( Build );

    /* Reset the target position.  If there's a collision during the next frame this will be undone. */
    PhysicsTarget = PhysicsClosed;

}

defaultproperties
{

    /* Add a second StaticMeshComponent with the standard bumper base plate.      */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=DynamicLightEnvironmentComponent Name=BaseLighting

        bEnabled = true;

    End Object

    Components.Add( BaseLighting );

    Begin Object Class=StaticMeshComponent Name=StaticMeshComponent1

        StaticMesh = StaticMesh'JWPContent.BumperBase';

        LightEnvironment = BaseLighting;
        bUsePrecomputedShadows = false;

    End Object

    Components.Add( StaticMeshComponent1 );

    /* Change the sound to a metallic "ping" noise more akin to a bumper.         */
    /*----------------------------------------------------------------------------*/
    Begin Object Name=AudioComponent0

        SoundCue = SoundCue'JWPContent.Ping';

    End Object

    /* Set the physics up to use the standard 32uu of travel and react quickly.   */
    /*----------------------------------------------------------------------------*/
    PhysicsDamping = 0.00001f;
    PhysicsOpen = -30.0f;
    PhysicsClosed = -2.0f;
    PhysicsCurrent = -2.0f;
    PhysicsTarget = -2.0f;
    PhysicsMinimum = -32.0f;
    PhysicsMaximum = 0.0f;
    PhysicsLimitbounce = 0.3f;
    PhysicsVelocity = 0.0f;
    PhysicsSpeed = 240.0f;

    /* Set the default points count to something sensible. */
    Points = 10;

}
