class JWP_Gameinfo extends Gameinfo DLLBind(HighscoreInterface);

/* How many balls does the player start with? */
var int StartingBalls;

/* How many points are required to score an extra ball? */
var int PointsPerBall;

/* How many balls are currently on the field? */
var int BallsInPlay;

/* How many points has the player amassed during this game? */
var int Points;

/* How many balls do they have left? */
var int Balls;

/* Whether an attempt to score points was made this frame. */
var bool ScoreDebounce;

/* Whether the attempts ceased and the countdown began. */
var bool ScoreDebounceOld;

/* How long left until we can score points again? */
var float ScoreDebounceRemaining;

/* Prevents a multiball being awarded more than once per turn. */
var bool MultiballBlock;

/* Inhibits BallDestroyed to prevent playfield clearing changing the game state. */
var bool InhibitBallDestroyed;

/* Which choice has been selected on the pause screen?  Resume/Restart/Quit. */
var int PauseOption;

/* No or yes on the restart/quit screens. */
var bool QuitOption;

/* Current highscore. */
var int Highscore;

dllimport final function WriteHighscore( int WriteThis );
dllimport final function int ReadHighscore();

/* The beginning state in which the title cards are shown.                    */
/*----------------------------------------------------------------------------*/
auto state Intro
{

    Begin:

    Highscore = ReadHighscore();

    ClearState();

    Balls = 0;
    Points = 0;

    BallsInPlay = 0;
    ScoreDebounce = false;
    ScoreDebounceOld = false;
    ScoreDebounceRemaining = 0.0f;
    MultiballBlock = false;

    ScoreboardDrawIntro();

    Sleep( 1.5f );

    ScoreboardDrawTitle();

    Sleep( 1.5f );

    Balls = StartingBalls;

    SpawnBall( 0 );

    Balls--;

    ScoreboardDrawHighscore();

    Sleep( 1.5f );

    ScoreboardDrawExtraBalls();

    Sleep( 1.5f );

    GotoState( 'Play' );

}

/* A test state for displaying all available scoreboard states.               */
/*----------------------------------------------------------------------------*/
state ScoreboardTest
{

    Begin:

    ScoreboardDrawIntro();

    Sleep( 1.5f );

    ScoreboardDrawTitle();

    Sleep( 1.5f );

    ScoreboardDrawExtraBalls();

    Sleep( 1.5f );

    ScoreboardDrawExtraBall();

    Sleep( 1.5f );

    ScoreboardDrawBonus( 1000 );

    Sleep( 1.5f );

    ScoreboardDraw();

    Sleep( 1.5f );

    PauseOption = 0;

    ScoreboardDrawPause();

    Sleep( 1.5f );

    PauseOption = 1;

    ScoreboardDrawPause();

    Sleep( 1.5f );

    PauseOption = 2;

    ScoreboardDrawPause();

    Sleep( 1.5f );

    PauseOption = 0;

    ScoreboardDrawAreYouSure();

    Sleep( 1.5f );

    PauseOption = 1;

    ScoreboardDrawAreYouSure();

    Sleep( 1.5f );

    ScoreboardDrawHighscore();

    Sleep( 1.5f );

    ScoreboardDrawNewHighscore();

    Sleep( 1.5f );

    ScoreboardDrawGameOver();

    Sleep( 1.5f );

    GotoState( 'ScoreboardTest' );

}

/* Primary gameplay.                                                          */
/*----------------------------------------------------------------------------*/
state Play
{

    /* The player hit pause.                                                      */
    /*----------------------------------------------------------------------------*/
    exec function PauseButton()
    {

        PauseOption = 0;

        GotoState( 'Paused' );

    }

    Begin:

    CameraPlaying();

    UnpauseObjects();

    ScoreboardDraw();

}

/* Gameplay has ended but is not yet ready to begin anew.                     */
/*----------------------------------------------------------------------------*/
state GameOver
{

    Begin:

    CameraPaused();

    ClearTimer( 'ScoreboardDraw' );
    ScoreboardDrawGameOver();

    Sleep( 2.0f );

    if( Points > Highscore )
    {
    
        ScoreboardDrawNewHighscore();
	WriteHighscore( Points );
        Highscore = Points;

        Sleep( 1.5f );
    
    }

    GotoState( 'Intro' );

}

/* The game is playing, but paused.                                           */
/*----------------------------------------------------------------------------*/
state Paused
{

    /* The player hit pause.                                                      */
    /*----------------------------------------------------------------------------*/
    exec function PauseButton()
    {

        GotoState( 'Play' );

    }

    /* Called when a flipper is hit.                                              */
    /*----------------------------------------------------------------------------*/
    exec function PinballFlip( int Flipper )
    {

        if( Flipper == 1 )
        {

            PauseOption--;
 
            if( PauseOption == -1 ) PauseOption = 2;

        }
        else
        {

            PauseOption++;
 
            if( PauseOption == 3 ) PauseOption = 0;

        }

        ScoreboardDrawPause();

    }

    /* The player hit the plunger.                                                */
    /*----------------------------------------------------------------------------*/
    exec function PinballPlungerPull()
    {

        if( PauseOption == 0 ) GotoState( 'Play' ); else GotoState( 'QuitMenu' );

    }

    Begin:

    CameraPaused();

    PauseObjects();

    ClearTimer( 'ScoreboardDraw' );
    ScoreboardDrawPause();

}

/* On the quit menu.                                                          */
/*----------------------------------------------------------------------------*/
state QuitMenu
{

    /* The player hit pause.                                                      */
    /*----------------------------------------------------------------------------*/
    exec function PauseButton()
    {

        GotoState( 'Paused' );

    }

    /* Called when a flipper is hit.                                              */
    /*----------------------------------------------------------------------------*/
    exec function PinballFlip( int Flipper )
    {

        QuitOption = !QuitOption;

        ScoreboardDrawAreYouSure();

    }

    /* The player hit the plunger.                                                */
    /*----------------------------------------------------------------------------*/
    exec function PinballPlungerPull()
    {

        local JWP_PlayerController Target;

        if( QuitOption ){ if( PauseOption == 1 ) Reset(); else foreach AllActors( class'JWP_PlayerController', Target ) Target.Quit(); } else GotoState( 'Paused' );

    }

    Begin:

    QuitOption = false;

    ScoreboardDrawAreYouSure();

}

/* Tells all TableObjects to not update their physics and freezes Kactors.    */
/*----------------------------------------------------------------------------*/
function PauseObjects()
{

    local JWP_TableObject Target;
    local JWP_Ball Freeze;

    foreach AllActors( class'JWP_TableObject', Target ) Target.Paused = true;

    foreach AllActors( class'JWP_Ball', Freeze ) Freeze.Pause();

}

/* Undoes PauseObjects.                                                       */
/*----------------------------------------------------------------------------*/
function UnpauseObjects()
{

    local JWP_TableObject Target;
    local JWP_Ball Thaw;

    foreach AllActors( class'JWP_TableObject', Target ) Target.Paused = false;

    foreach AllActors( class'JWP_Ball', Thaw ) Thaw.Unpause();

}


/* Moved from the PlayerController.  Send signals to the table objects.       */
/*----------------------------------------------------------------------------*/
exec function PinballPlungerPull()
{

    local JWP_Plunger Target;

    foreach AllActors( class'JWP_Plunger', Target ) Target.Pull();

}

exec function PinballPlungerRelease()
{

    local JWP_Plunger Target;

    foreach AllActors( class'JWP_Plunger', Target ) Target.Release();

}

exec function PinballFlip( int FlipID )
{

    local JWP_Flipper Target;

    foreach AllActors( class'JWP_Flipper', Target ) Target.Flip( FlipID );

}

    exec function PinballUnflip( int FlipID )
    {

        local JWP_Flipper Target;

        foreach AllActors( class'JWP_Flipper', Target ) Target.Unflip( FlipID );

    }


/* Called by anything that might trigger multiball.  Checks if it hasn't      */
/* happened this life yet and if not spawns the other two balls.              */
/*----------------------------------------------------------------------------*/
function Multiball()
{

    if( !MultiballBlock )
    {

        SpawnBall( 1 );
        SpawnBall( 2 );

        MultiballBlock = true;

    }

}

/* If there are no inactive switches in the scene, unflips all switches and   */
/* awards a points bonus.                                                     */
/*----------------------------------------------------------------------------*/
function Switches()
{

    local JWP_Switch Target;

    local bool Bad;

    Bad = false;

    foreach AllActors( class'JWP_Switch', Target ) if( !Target.Swap ) Bad = true;

    if( !Bad )
    {

        foreach AllActors( class'JWP_Switch', Target ) Target.Unflip();

        ForceScorePoints( 1000 );

        ScoreboardDrawBonus( 1000 );

    }

}

/* Destroys all balls in the scene.  Used by the reset path.                  */
/* Sets a bool that disables BallDestroyed to prevent state corruption.       */
/* Also unflips all switches.                                                 */
/*----------------------------------------------------------------------------*/
function ClearState()
{

    local JWP_Ball Target;
    local JWP_Switch Unflip;

    InhibitBallDestroyed = true;

    foreach AllActors( class'JWP_Ball', Target ) Target.Destroy();
    foreach AllActors( class'JWP_Switch', Unflip ) Unflip.Unflip();

    InhibitBallDestroyed = false;

}

/* Decides on an action when a ball is lost.                                  */
/*----------------------------------------------------------------------------*/
function BallDestroyed()
{

    BallsInPlay--;

    if( !InhibitBallDestroyed )
    {

        if( BallsInPlay == 0 )
        {

            if( Balls == 0 )
            {

                GotoState( 'GameOver' );

            }
            else
            {

                MultiballBlock = false;

                SpawnBall( 0 );

                Balls--;

                if( !IsTimerActive( 'ScoreboardDraw' ) ) ScoreboardDraw();

            }

        }

    }

}

/* Iterates through the ball factories with the correct ID, chooses a random  */
/* one and uses its position to spawn a ball.  Used to work via plungers.     */
/*----------------------------------------------------------------------------*/
function SpawnBall( byte BallID )
{

    local JWP_BallFactory Target;
    local int Spawns;
    local vector Position;

    Super.PostBeginPlay();

    Spawns = 0;

    foreach AllActors( class'JWP_BallFactory', Target ) if( Target.BallID == BallID ) Spawns++;

    if( Spawns > 0 )
    {

        Spawns = rand( Spawns );

        Position.X = 0.0f;
        Position.Y = 0.0f;
        Position.Z = 0.0f;

        foreach AllActors( class'JWP_BallFactory', Target )
        {

            if( Target.BallID == BallID )
            {

                Position = Target.Location;

                if( Spawns == 0 ) break;

                Spawns--;

            }

        }

        Spawn( class'JWP_Ball', , , Position, , , false );

        BallsInPlay++;

    }
    else
    {

        BallsInPlay++;
        BallDestroyed();

    }

}

/* An externally callable function that clears the notification timer and     */
/* resets to the initial state.                                               */
/*----------------------------------------------------------------------------*/
Function Reset()
{

    ClearTimer( 'ScoreboardDraw' );

    GotoState( 'Intro' );

}

/* Moves all cameras to looking down on the board.                            */
/*----------------------------------------------------------------------------*/
function CameraPlaying()
{

    local JWP_PlayerController Target;

    foreach AllActors( class'JWP_PlayerController', Target ) Target.CameraPlaying();

}

/* Moves all cameras to looking at the scoreboard.                             */
/*----------------------------------------------------------------------------*/
function CameraPaused()
{

    local JWP_PlayerController Target;

    foreach AllActors( class'JWP_PlayerController', Target ) Target.CameraPause();

}

/* Displays the high score screen.                                            */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawHighScore()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawHighscore( Highscore );

}

/* Informs the player of their new high score.                                */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawNewHighScore()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawNewHighscore( Highscore );

}

/* Shows the pause screen.                                                    */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawPause()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawPause( PauseOption );

}

/* Asks the player if they are certain they want to quit.                     */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawAreYouSure()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawAreYouSure( QuitOption );

}

/* Pushes a GAME OVER message to all scoreboards in the game world.           */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawGameOver()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawGameOver();

    ClearTimer( 'ScoreboardDraw' );

}

/* Tell the player they've recieved a points bonus via the scoreboard.        */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawBonus( int Bonus )
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawBonus( Bonus );

    /* This should override balls/points updates so they will not update the display for another 1.5f seconds. */
    ClearTimer( 'ScoreboardDraw' );
    SetTimer( 1.5f, false, 'ScoreboardDraw' );

}

/* Notifies player via scoreboards of being awarded an extra ball.            */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawExtraBall()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawExtraBall( Balls );

    /* This should override balls/points updates so they will not update the display for another 1.5f seconds. */
    ClearTimer( 'ScoreboardDraw' );
    SetTimer( 1.5f, false, 'ScoreboardDraw' );

}

/* Displays a notification of multiball being awarded on scoreboards.         */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawMultiBall()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawMultiBall( Balls );

    /* We want this to hang around on the display for a little while so this prevents everything but other notifications from drawing for 1.5f seconds. */
    ClearTimer( 'ScoreboardDraw' );
    SetTimer( 1.5f, false, 'ScoreboardDraw' );

}

/* Views the introduction graphic on all scoreboards.                         */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawIntro()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawIntro();

}

/* Shows the title screen graphic on the scoreboards.                         */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawTitle()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawTitle();

}

/* Displays the amount of points needed for an extra ball on all scoreboards. */
/*----------------------------------------------------------------------------*/
function ScoreboardDrawExtraBalls()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawExtraBalls( PointsPerBall );

}

/* Draws the ball count and score to every scoreboard in the game world.      */
/*----------------------------------------------------------------------------*/
function ScoreboardDraw()
{

    local JWP_Scoreboard Target;

    Super.PostBeginPlay();

    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawBalls( Balls );
    foreach AllActors( class'JWP_Scoreboard', Target ) Target.DrawPoints( Points );

}

/* Called by any object that can score points.                                */
/* NOTE: Inbuilt debounce logic MAY cause point scoring to fail, use          */
/* ForceScorePoints to bypass the debounce mechanism for points awarded by    */
/* special events/etc. that should not be missed.                             */
/*----------------------------------------------------------------------------*/
function ScorePoints( int AddPoints )
{

    if( ScoreDebounceRemaining < 0.0f )
    {

        ForceScorePoints( AddPoints );

        ScoreDebounce = true;
        ScoreDebounceOld = true;

        ScoreDebounceRemaining = 0.1f;

    }

}

/* Bypasses the point scoring debounce but scores points like ScorePoints.    */
/*----------------------------------------------------------------------------*/
function ForceScorePoints( int AddPoints )
{

    local int OldPoints;

    OldPoints = Points;

    Points += AddPoints;
    if( Points > 9999999 ) Points = 9999999;

    if( ( Points / PointsPerBall ) > ( OldPoints / PointsPerBall ) )
    {

        Balls += ( Points / PointsPerBall ) - ( OldPoints / PointsPerBall );

        ScoreboardDrawExtraBall();

    }

    if( !IsTimerActive( 'ScoreboardDraw' ) ) ScoreboardDraw();

}

/* Ran every frame.                                                           */
/*----------------------------------------------------------------------------*/
event Tick( float DeltaTime )
{

    /* Perform any tasks required by the superclass. */
    Super.Tick( DeltaTime );

    /* Code for debouncing the score - the score counter will be static until the */
    /* attempts to change it stop, then there is a 0.1f second delay before it    */
    /* becomes a modifiable value again.                                          */
    /*----------------------------------------------------------------------------*/
    if( ( !ScoreDebounce ) && ScoreDebounceOld )
    {

         ScoreDebounceOld = false;

    }

    if( !ScoreDebounceOld )
    {

        ScoreDebounceRemaining -= DeltaTime;

    }

    ScoreDebounce = false;
    /*----------------------------------------------------------------------------*/

}

exec function TestDisplay()
{

    GotoState( 'ScoreboardTest' );

}

defaultproperties
{

    /* Select our custom player controller for use in this game mode. */
    PlayerControllerClass = class'JWP_PlayerController';

    StartingBalls = 3;
    PointsPerBall = 2000;

    Balls = 0;
    Points = 0;

    BallsInPlay = 0;
    ScoreDebounce = false;
    ScoreDebounceOld = false;
    ScoreDebounceRemaining = 0.0f;
    MultiballBlock = false;
    InhibitBallDestroyed = false;

    PauseOption = 0;
    QuitOption = 0;
    Highscore = 0;

}
