class JWP_Plunger extends JWP_TableObject placeable ClassGroup( Pinball );

/* The physics in JWP_TableObject are largely reusable for this instance      */
/* except we need separate settings for pulling and releasing the plunger.    */
var float PullDamping;
var float ReleaseDamping;
var float PullBounce;
var float ReleaseBounce;

/* A reference to the mesh component for the spring so we can scale it.       */
var StaticMeshComponent Springmesh;

/* Called by JWP_PlayerController to tell the plunger to move.                */
/*----------------------------------------------------------------------------*/
function Pull()
{

    /* Set the physics up to pull the plunger back slowly. */
    PhysicsDamping = PullDamping;
    PhysicsLimitbounce = PullBounce;
    PhysicsTarget = PhysicsOpen;

}

function Release()
{

    /* Set the physics up to push the plunger forwards quickly. */
    PhysicsDamping = ReleaseDamping;
    PhysicsLimitbounce = ReleaseBounce;
    PhysicsTarget = PhysicsClosed;

}
/*----------------------------------------------------------------------------*/

/* Event is executed every frame.                                             */
/*----------------------------------------------------------------------------*/
simulated event Tick( float DeltaSeconds )
{

    /* Temporary variables to store the new position of the plunger and scale of the spring. */
    local vector Position;
    local vector Scale;

    /* Call our superclass's Tick() function so it can update the physics. */
    Super.Tick( DeltaSeconds );

    Position.x = PhysicsCurrent;
    Position.y = 0.0f;
    Position.z = 0.0f;

    /* Normalise the plunger position to 0.0f ... 1.0f as a scale for the spring. */
    Scale.X = 1.0f - ( ( PhysicsCurrent - PhysicsClosed ) / ( PhysicsOpen - PhysicsClosed ) );
    Scale.Y = 1.0f;
    Scale.Z = 1.0f;

    /* Set the position of the plunger and the scale of the spring. */
    StaticMeshComponent.SetTranslation( Position );
    Springmesh.SetScale3D( Scale );

}

defaultproperties
{

    /* Reusing DynamicSMActor's StaticMeshComponent as the plunger. */
    Begin Object Name=StaticMeshComponent0

        StaticMesh = StaticMesh'JWPContent.PlungerPlunger';

    End Object

    /* Add a StaticMeshComponent with dynamic lighting for the body.              */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=DynamicLightEnvironmentComponent Name=BodyLighting

        bEnabled = true;

    End Object

    Components.Add( BodyLighting );

    Begin Object Class=StaticMeshComponent Name=StaticMeshComponent1

        StaticMesh = StaticMesh'JWPContent.PlungerBody';

        LightEnvironment = BodyLighting;
        bUsePrecomputedShadows = false;

    End Object

    Components.Add( StaticMeshComponent1 );

    /* Add a StaticMeshComponent with dynamic lighting for a spring.              */
    /*----------------------------------------------------------------------------*/
    /* Make a separate lighting environment for the spring. */
    Begin Object Class=DynamicLightEnvironmentComponent Name=SpringLighting

        bEnabled = true;

    End Object

    Components.Add( SpringLighting );

    /* Make our static mesh component using the lighting environment we just      */
    /* created and the spring mesh.                                               */
    Begin Object Class=StaticMeshComponent Name=StaticMeshComponent2

        StaticMesh = StaticMesh'JWPContent.PlungerSpring';
        LightEnvironment = SpringLighting;
        bUsePrecomputedShadows = false;

    End Object

    /* Keep a reference to it so we can scale it later. */
    Springmesh = StaticMeshComponent2;

    Components.Add( StaticMeshComponent2 );
    /*----------------------------------------------------------------------------*/

    /* Plunger-specific physics variables used for altering the generic physics   */
    /* settings depending upon state.                                             */
    PullDamping = 0.8f;
    ReleaseDamping = 0.00000000001f;
    ReleaseBounce = 0.3f;
    PullBounce = 0.025f;

    /* Default physics settings. */
    PhysicsOpen = 336.0f;
    PhysicsClosed = 0.0f;
    PhysicsMinimum = 0.0f;
    PhysicsMaximum = 336.0f;
    PhysicsDamping = 0.00000000001f;
    PhysicsLimitbounce = 0.3f;
    PhysicsTarget = 0.0f;
    PhysicsSpeed = 20.0f;

}
