class JWP_Switch extends DynamicSMActor placeable ClassGroup( Pinball );

var bool Swap;
var bool Flipped;
var bool Debouncing;
var float Debounce;
var LensFlareComponent Indicator;
var PointLightComponent Glare;

var AudioComponent FlipSound;
var AudioComponent UnflipSound;

var bool FlipSoundPlayed;
var bool UnflipSoundPlayed;

Function Flip()
{

    local color Colour;

    Swap = true;

    Colour.R = 0;
    Colour.G = 255;
    Colour.B = 0;
    Colour.A = 0;

    Indicator.SetTemplate( LensFlare'JWPContent.SwitchGreen' );
    Glare.SetLightProperties( 2.0f, Colour );

    if( FlipSound.bFinished || ( !FlipSoundPlayed ) )
    {

        FlipSound.Stop();
        FlipSound.Play();

        FlipSoundPlayed = true;

    }    
   
    JWP_Gameinfo( WorldInfo.Game ).Switches();

}

Function UnFlip()
{

    local color Colour;

    Swap = false;

    Colour.R = 255;
    Colour.G = 0;
    Colour.B = 0;
    Colour.A = 0;

    Indicator.SetTemplate( LensFlare'JWPContent.SwitchRed' );
    Glare.SetLightProperties( 2.0f, Colour );   

}

Event Tick( float DeltaFrames )
{

    local JWP_Ball Find;
    local bool Hit;

    Super.Tick( DeltaFrames );

    Hit = false;

    Foreach OverlappingActors( class'JWP_Ball', Find, 88.0f ) Hit = true;

    if( Debouncing )
    {

        Debounce -= DeltaFrames;

        if( Debounce < 0.0f ){ Debouncing = false; Flipped = false; }

    }

    if( Hit )
    {

        if( !Flipped ) if( Swap ) Unflip(); else
        {

            Flip();

            if( UnflipSound.bFinished || ( !UnflipSoundPlayed ) )
            {

                UnflipSound.Stop();
                UnflipSound.Play();

                UnflipSoundPlayed = true;

            } 

        }

        Flipped = true;

    }
    else
    {

        if( Flipped ){ Debouncing = true; Debounce = 0.1f; }

    }

}

defaultproperties
{

    /*----------------------------------------------------------------------------*/
    Begin Object Name=StaticMeshComponent0

        StaticMesh = StaticMesh'JWPContent.Switch';

    End Object
    /*----------------------------------------------------------------------------*/

    /* An AudioComponent for when the switch is triggered.                        */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=AudioComponent Name=AudioComponent0

        bAutoPlay = false;
        bStopWhenOwnerDestroyed = true;
        bShouldRemainActiveIfDropped = true;

        SoundCue = SoundCue'JWPContent.SwitchingOn';

    End Object

    FlipSound = AudioComponent0;

    Components.Add( AudioComponent0 );
    /*----------------------------------------------------------------------------*/


    /* An AudioComponent for when the switch is untriggered.                      */
    /*----------------------------------------------------------------------------*/
    Begin Object Class=AudioComponent Name=AudioComponent1

        bAutoPlay = false;
        bStopWhenOwnerDestroyed = true;
        bShouldRemainActiveIfDropped = true;

        SoundCue = SoundCue'JWPContent.SwitchingOff';

    End Object

    UnflipSound = AudioComponent1;

    Components.Add( AudioComponent1 );
    /*----------------------------------------------------------------------------*/

    Begin Object Class=PointLightComponent Name=PointLightComponent0

	Brightness = 2.0f;
	Radius = 192.0f;
	CastShadows = false;
	LightColor = ( R = 255, G = 0, B = 0, A = 255 );
        Translation = ( X = 0.0f, Y = 0.0f, Z = 32.0f );

    End Object

    Glare = PointLightComponent0;

    Components.Add( PointLightComponent0 );

    Begin Object Class=LensFlareComponent Name=LensFlareComponent0

	Template = LensFlare'JWPContent.SwitchRed';
        Translation = ( X = 0.0f, Y = 0.0f, Z = 32.0f );

    End Object

    Indicator = LensFlareComponent0;

    Components.Add( LensFlareComponent0 );

    FlipSoundPlayed = false;
    UnflipSoundPlayed = false;

    Swap = false;
    Flipped = false;
    Debounce = 0.0f;
    Debouncing = false;
   
}
